@@ -22,11 +22,3 @@ impl embedded_hal_0_2::adc::Channel<adc::ADC1> for HallSensor {
         ()
     }
 }
-
-impl embedded_hal::adc::nb::Channel<adc::ADC1> for HallSensor {
-    type ID = ();
-
-    fn channel(&self) -> Self::ID {
-        ()
-    }
-}
